<!-- field/annalist_edit_identifier.html -->
<input type="text" size="64" name="{{repeat_prefix}}{{field.field_name}}" 
       placeholder="{{field.field_placeholder}}"
       value="{{field.field_value}}"/>